// Unreal Fighting Engine by Kevin Poretti

#pragma once

#include "CoreMinimal.h"
#include "UObject/Interface.h"

#include "IFESystemInterface.generated.h"

// This class does not need to be modified.
UINTERFACE(MinimalAPI)
class UIFESystemInterface : public UInterface
{
	GENERATED_BODY()
};

/**
 *	This interface defines functions that need to be implemented for any objects that are "gameplay affecting".
 *
 *	This ensures all gameplay effecting objects can be assumed to have certain properties that enable serialization, networking and
 *	some form of determinism.
 */
class UNREALFIGHTINGENGINE_API IIFESystemInterface
{
	GENERATED_BODY()

public:
	/**
	 *	Function to be called at a fixed interval/frame rate rather than using Unreal's variable Tick function
	 *
	 *	@param OneFrame the time that elapses during one fixed tick
	 */
    virtual void FixedTick(float OneFrame) = 0;
};
